@@ -3,7 +3,7 @@
 @Time   : 2019/11/25 13:15
 @Author : LeeCQ
 
-[MySQL(pymysql, 需要服务器),  MongoDB(pyMongoDB，需要服务器),
+[mysql(pymysql, 需要服务器),  MongoDB(pyMongoDB，需要服务器),
  SQLite(关系型数据库, Python原生支持),
  bsddb(NoSQL Bsddb3),
  UnQLite(NoSQL),
@@ -14,7 +14,7 @@
 
 ]
 
-MySQL:
+mysql:
 
 SQLite:
 
@@ -32,13 +32,13 @@ LiteDB:
 """
 
 from .SQLite import SQLiteAPI
-from .MySQL import MyMySqlAPI, MySqlAPI
-from .SQLCommon import sql_join
-
-# __import__('SQLCommon')
+from .mysql import MyMySqlAPI, MySqlAPI
+from .common.common import sql_join
+from .common.base_sql import BaseSQL, BaseSQLAPI
+from .common import common
 
 # 直接访问会出错，但是，其他模块可以正常导入这些API
-__version__ = '0.2.6'
+__version__ = '0.2.6.2'
 
-__all__ = ['SQLCommon',
-           'SQLiteAPI', 'MyMySqlAPI', 'MySqlAPI', 'sql_join']
+# __all__ = ['common', 'BaseSQL',
+#            'SQLiteAPI', 'MyMySqlAPI', 'MySqlAPI', 'sql_join']
